"plotPostP" <-
function(postP, x, p, step, dat.col="orange", fit.col="seagreen"){

    "get.end" <- 
    function(xk, pk, p.end, state, x.fix, p.fix, b.limit=1/1000){
        if(state==1 || state==2){
            end = x.fix
        }else{
            ## linear model is: x-x.fix = b(p - p.fix)
            nxk = xk - x.fix
            npk = pk - p.fix
            s = sum(nxk*npk)/sum(npk*npk)
            b = s
            if(s<=0){
                if(state==4){ end = NA }
                else{
                    if(s > -b.limit){ b = -b.limit }
                    end = x.fix + b*(p.end-p.fix)
                }
            }else{
                if(state==3){ end = NA }
                else{
                    if(s < b.limit){ b = b.limit }
                    end = x.fix + b*(p.end-p.fix)
                }
            }
        }
        return(end)
    }

    postS = apply(postP, 1, which.max)
    postS = as.numeric(postS)
    
    Z = nrow(postP)

    ## step indicator, every 'step' bp is one step
    stp = ceiling(p/step)
    
    ## check that for the input data, every step have at least 3 observations
    len = tapply(p, stp, length)
    if(any(len<3)){
        stop("there is at least one step with less than 3 observations\n")
    }
    
    ## indicator of end of every step
    ## minus 1 to mach the index in c
    dif = diff(stp)
    stp.end = c(which(dif>0), length(stp))

    ## make sure p start from a value < step
    p = p - floor(min(p)/step)*step

    min.x = min(x)
    max.x = max(x)
    rag.x = max.x - min.x
    
    mm = "hidden states based posterior probability"
    plot(p, x, ylim = c(min.x-0.2*rag.x, max.x), type="l", col=dat.col, 
        xlab = "position", ylab="signal", lwd=1.5, main=mm)
        
    xS = step*(1:Z) - step/2
    
    states = unique(postS)
    rbcols = rainbow(length(states))
    for(i in 1:length(states)){
        st = states[i]
        xI = xS[postS==st]
        yI = rep(min.x-0.1*rag.x, length(xI))
        points(xI, yI, pch=as.character(st), col=rbcols[i])
    }
    
    path.end = c(which(diff(postS)!=0), Z)
    path.sta = postS[path.end]
    
    
    for(i in 1:length(path.end)){
        if(i==1){
            start = 1
            kp.start = 1
        }else{
            start = path.end[i-1]+1
            kp.start = stp.end[start-1]+1
        }
        
        end   = path.end[i]
        
        kp = kp.start:stp.end[end]
        xk = x[kp]
        pk = p[kp]
        
        xf = mean(xk)
        pf = mean(pk)
        
        p1 = step*(start-1) + 1
        p2 = step*end
        
        x1 = get.end(xk, pk, p1, path.sta[i], xf, pf)
        x2 = get.end(xk, pk, p2, path.sta[i], xf, pf)
        
        segments(p1, x1, p2, x2, col=fit.col, lty=2, lwd=2)
    }
}

