# -------------------------------------------------------------------------- #
# get the end of each psible sequence segment
#
# -------------------------------------------------------------------------- #

"get.ends" <- 
function(x, p, step, b.limit){
  
    ## make sure p start from a value < step
    p = p - floor(min(p)/step)*step
    
    ## how many steps altogether
    D1 = ceiling(max(p)/step)

    ## block indicator, every 'step' bp is one block
    blk = ceiling(p/step)
    
    ## check that for the input data, every block have at least 2 observations
    len = tapply(p, blk, length)
    if(any(len<2)){
        stop("there is at least one block with less than 2 observations\n")
    }
    
    ## indicator of end of every block
    dif = diff(blk)
    blk.end = c(which(dif>0), length(blk))
    
    ## end of every block: x1.end = x2.end
    x1.end = x2.end = x3.end = x4.end = matrix(NA, nrow=D1, ncol=D1)
    
    i = 1
    for(j in i:D1){
        kp  = 1:blk.end[j]
        end = get.end(x[kp], p[kp], step*j, b.limit=b.limit)
        x1.end[1,j] = end[1]
        x2.end[1,j] = end[2]
        x3.end[1,j] = end[3]
        x4.end[1,j] = end[4]
    }
    
    for(i in 2:D1){
        for(j in i:D1){
            kp  = (blk.end[i-1]+1):blk.end[j]
            end = get.end(x[kp], p[kp], step*j, b.limit=b.limit)
            x1.end[i,j] = end[1]
            x2.end[i,j] = end[2]
            x3.end[i,j] = end[3]
            x4.end[i,j] = end[4]
        }
    }
    
    list(x1.end, x2.end, x3.end, x4.end)
}